package net.knarcraft.dynmapcitizens.handler.trait.quests;

import me.pikamug.quests.Quests;
import me.pikamug.quests.quests.Quest;
import me.pikamug.quests.quests.components.Stage;
import net.knarcraft.dynmapcitizens.DynmapCitizens;
import net.knarcraft.dynmapcitizens.settings.QuestsSettings;
import net.knarcraft.dynmapcitizens.util.DynmapHelper;
import net.knarcraft.dynmapcitizens.util.QuestsHelper;
import net.knarcraft.knarlib.formatting.StringFormatter;
import net.knarcraft.knarlib.formatting.StringReplacer;
import net.knarcraft.knarlib.formatting.Translator;
import org.bukkit.Location;
import org.bukkit.entity.EntityType;
import org.dynmap.DynmapAPI;
import org.dynmap.markers.GenericMarker;
import org.dynmap.markers.MarkerSet;
import org.jetbrains.annotations.NotNull;

import java.util.List;

import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_KILL_AREA_DESCRIPTION_FORMAT;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_KILL_AREA_NAME_FORMAT;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REACH_AREA_DESCRIPTION_FORMAT;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REACH_AREA_NAME_FORMAT;

/**
 * A handler class for quest areas
 */
public class QuestAreaHandler {

    private final Quests questsAPI;
    private final MarkerSet killAreaMarkerSet;
    private final MarkerSet reachAreaMarkerSet;
    private final QuestsSettings settings;
    private final List<Quest> unavailableQuests;
    private final StringFormatter formatter;

    /**
     * Instantiates a new quest area handler
     *
     * @param questsAPI         <p>A reference to the quests API</p>
     * @param dynmapAPI         <p>A reference to the dynmap API</p>
     * @param settings          <p>The quests settings to use</p>
     * @param unavailableQuests <p>The list of currently unavailable quests to possibly skip</p>
     */
    public QuestAreaHandler(@NotNull Quests questsAPI, @NotNull DynmapAPI dynmapAPI, @NotNull QuestsSettings settings,
                            @NotNull List<Quest> unavailableQuests) {
        this.questsAPI = questsAPI;
        this.settings = settings;
        this.unavailableQuests = unavailableQuests;
        this.formatter = DynmapCitizens.getFormatter();
        killAreaMarkerSet = DynmapHelper.initializeMarkerSet(dynmapAPI, settings.getKillAreaSettings());
        reachAreaMarkerSet = DynmapHelper.initializeMarkerSet(dynmapAPI, settings.getReachAreaSettings());
    }

    /**
     * Updates all quest area markers
     */
    public void updateQuestAreas() {
        this.killAreaMarkerSet.getCircleMarkers().forEach(GenericMarker::deleteMarker);
        this.reachAreaMarkerSet.getCircleMarkers().forEach(GenericMarker::deleteMarker);
        for (Quest quest : questsAPI.getLoadedQuests()) {
            //Skip marker if quest is unavailable, and unavailable quests are hidden
            if (settings.hideUnavailableQuests() && this.unavailableQuests.contains(quest)) {
                continue;
            }
            for (Stage stage : quest.getStages()) {
                markKillLocations(quest, stage);
                markReachLocations(quest, stage);
            }
        }
        //TODO: Mark WorldGuard areas part of quests. Requires WorldGuard integration
    }

    /**
     * Marks any reach locations found in the given stage
     *
     * @param quest <p>The quest the stage belongs to</p>
     * @param stage <p>The stage to search for reach locations</p>
     */
    private void markReachLocations(@NotNull Quest quest, @NotNull Stage stage) {
        if (settings.getReachAreaSettings().isDisabled()) {
            return;
        }
        for (int i = 0; i < stage.getLocationsToReach().size(); i++) {
            Location location = (Location) stage.getLocationsToReach().get(i);
            int radius = stage.getRadiiToReachWithin().get(i);
            String areaName = stage.getLocationNames().get(i);

            String formattedAreaName;
            if (areaName != null) {
                formattedAreaName = formatter.replacePlaceholder(QUESTS_REACH_AREA_NAME_FORMAT, "{name}", areaName);
            } else {
                formattedAreaName = "";
            }

            String description = formatter.replacePlaceholders(QUESTS_REACH_AREA_DESCRIPTION_FORMAT,
                    new String[]{"{areaName}", "{questName}"}, new String[]{formattedAreaName, quest.getName()});
            DynmapHelper.markLocation(location, radius, description, reachAreaMarkerSet, settings.getReachAreaSettings());
        }
    }

    /**
     * Marks any kill locations found in the given stage
     *
     * @param quest <p>The quest the stage belongs to</p>
     * @param stage <p>The stage to search for kill locations</p>
     */
    private void markKillLocations(@NotNull Quest quest, @NotNull Stage stage) {
        if (settings.getKillAreaSettings().isDisabled()) {
            return;
        }
        for (int i = 0; i < stage.getLocationsToKillWithin().size(); i++) {
            Location location = (Location) stage.getLocationsToKillWithin().get(i);
            int radius = stage.getRadiiToKillWithin().get(i);
            EntityType mob = (EntityType) stage.getMobsToKill().get(i);
            int mobAmount = stage.getMobNumToKill().get(i);
            String areaName = stage.getKillNames().get(i);

            String formattedAreaName;
            if (areaName != null) {
                formattedAreaName = formatter.replacePlaceholder(QUESTS_KILL_AREA_NAME_FORMAT, "{name}", areaName);
            } else {
                formattedAreaName = "";
            }

            Translator translator = DynmapCitizens.getTranslator();
            StringReplacer replacer = new StringReplacer(translator.getTranslatedMessage(QUESTS_KILL_AREA_DESCRIPTION_FORMAT));
            replacer.add("{areaName}", formattedAreaName);
            replacer.add("{questName}", quest.getName());
            replacer.add("{mobName}", QuestsHelper.normalizeName(mob.name()));
            replacer.add("{mobAmount}", String.valueOf(mobAmount));
            DynmapHelper.markLocation(location, radius, replacer.replace(), killAreaMarkerSet, settings.getKillAreaSettings());
        }
    }

}
