package net.knarcraft.dynmapcitizens.handler.trait.quests;

import me.pikamug.quests.Quests;
import me.pikamug.quests.quests.Quest;
import me.pikamug.quests.quests.components.Requirements;
import net.knarcraft.dynmapcitizens.DynmapCitizens;
import net.knarcraft.dynmapcitizens.util.QuestsHelper;
import net.knarcraft.knarlib.formatting.StringFormatter;
import net.knarcraft.knarlib.formatting.StringReplacer;
import net.knarcraft.knarlib.formatting.TranslatableMessage;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REQUIREMENTS_BLOCKED_BY_QUEST_FORMAT;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REQUIREMENTS_BLOCKED_BY_QUEST_ITEM;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REQUIREMENTS_EXP;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REQUIREMENTS_FORMAT;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REQUIREMENTS_MC_MMO_SKILL;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REQUIREMENTS_QUEST_POINTS;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REQUIREMENTS_REQUIRED_ITEM_FORMAT;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REQUIREMENTS_REQUIRED_ITEM_ITEM;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REQUIREMENTS_REQUIRED_PERMISSION_FORMAT;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REQUIREMENTS_REQUIRED_PERMISSION_ITEM;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REQUIREMENTS_REQUIRED_QUEST_FORMAT;
import static net.knarcraft.dynmapcitizens.formatting.QuestsTranslatableMessage.QUESTS_REQUIREMENTS_REQUIRED_QUEST_ITEM;

/**
 * A class to generate a string containing all information about a quest's requirements
 */
public class QuestRequirementsInfoGenerator {

    private final Quests questsAPI;
    private final Quest quest;
    private final StringFormatter formatter;

    /**
     * Instantiates a new quest requirement info generator
     *
     * @param questsAPI <p>The API to use for getting quest information</p>
     * @param quest     <p>The quest to generate information about</p>
     */
    public QuestRequirementsInfoGenerator(@NotNull Quests questsAPI, @NotNull Quest quest) {
        this.questsAPI = questsAPI;
        this.quest = quest;
        formatter = DynmapCitizens.getFormatter();
    }

    /**
     * Gets information about all requirements for the given quest
     *
     * @return <p>Information about the quest's requirements</p>
     */
    public @NotNull String getQuestRequirementsInfo() {
        Requirements requirements = quest.getRequirements();
        if (!requirements.hasRequirement()) {
            return "";
        }

        StringReplacer replacer = new StringReplacer(DynmapCitizens.getTranslator().getTranslatedMessage(
                QUESTS_REQUIREMENTS_FORMAT));

        //Add info about quest point requirement
        replacer.add("{requirementQuestPoints}", requirements.getQuestPoints() > 0 ?
                formatter.replacePlaceholder(QUESTS_REQUIREMENTS_QUEST_POINTS,
                        "{questPoints}", String.valueOf(requirements.getQuestPoints())) : "");

        //Add info about exp requirement
        replacer.add("{requirementExp}", requirements.getExp() > 0 ? formatter.replacePlaceholder(
                QUESTS_REQUIREMENTS_EXP, "{exp}", String.valueOf(requirements.getExp())) : "");

        //Add info about blocking quests
        replacer.add("{requirementBlockedByQuests}", !requirements.getBlockQuestIds().isEmpty() ?
                getRequirementList(getQuestNames(requirements.getBlockQuestIds()),
                        QUESTS_REQUIREMENTS_BLOCKED_BY_QUEST_FORMAT, "{blockingQuests}",
                        QUESTS_REQUIREMENTS_BLOCKED_BY_QUEST_ITEM, "{questName}") : "");

        //Add info about required quests
        replacer.add("{requirementRequiredQuests}", !requirements.getBlockQuestIds().isEmpty() ?
                getRequirementList(getQuestNames(requirements.getBlockQuestIds()),
                        QUESTS_REQUIREMENTS_REQUIRED_QUEST_FORMAT, "{requiredQuests}",
                        QUESTS_REQUIREMENTS_REQUIRED_QUEST_ITEM, "{questName}") : "");

        //Add info about required items
        replacer.add("{requirementRequiredItems}", !requirements.getItems().isEmpty() ?
                getRequirementList(getItemNames(requirements.getItems()), QUESTS_REQUIREMENTS_REQUIRED_ITEM_FORMAT,
                        "{requiredItems}", QUESTS_REQUIREMENTS_REQUIRED_ITEM_ITEM, "{itemName}") : "");

        //Add info about required mcMMO skills
        if (!requirements.getMcmmoSkills().isEmpty()) {
            List<String> skills = requirements.getMcmmoSkills();
            List<Integer> amounts = requirements.getMcmmoAmounts();
            StringBuilder mcMMOSkillsBuilder = new StringBuilder();
            for (int i = 0; i < skills.size(); i++) {
                mcMMOSkillsBuilder.append(formatter.replacePlaceholders(QUESTS_REQUIREMENTS_MC_MMO_SKILL, new String[]{
                        "{skill}", "{level}"}, new String[]{skills.get(i), String.valueOf(amounts.get(i))}));
            }
            replacer.add("{requirementMCMMOSkills}", mcMMOSkillsBuilder.toString());
        } else {
            replacer.add("{requirementMCMMOSkills}", "");
        }

        //Add info about required permissions
        replacer.add("{requirementPermissions}", !requirements.getPermissions().isEmpty() ?
                getRequirementList(requirements.getPermissions(), QUESTS_REQUIREMENTS_REQUIRED_PERMISSION_FORMAT,
                        "{permissions}", QUESTS_REQUIREMENTS_REQUIRED_PERMISSION_ITEM, "{permission}") : "");

        Map<String, Map<String, Object>> customRequirementPlugins = requirements.getCustomRequirements();
        StringBuilder customRequirementsBuilder = new StringBuilder();
        for (String plugin : customRequirementPlugins.keySet()) {
            customRequirementsBuilder.append("<li>").append(plugin).append(":<ul>");
            //Note: The format of custom requirements is kind of weird. First, you have the key for which plugin the 
            // requirement belongs to. Getting the value of the key gives another map. The map contains as key, the type
            // of value, like "Skill Amount" or "Skill Type". The value is the actual value of whatever it is.
            Map<String, Object> customRequirementEntry = customRequirementPlugins.get(plugin);
            for (String requirementDescription : customRequirementEntry.keySet()) {
                customRequirementsBuilder.append("<li>").append(requirementDescription).append(" ");
                customRequirementsBuilder.append(customRequirementEntry.get(requirementDescription)).append("</li>");
            }
            customRequirementsBuilder.append("</ul></li>");
        }
        replacer.add("{requirementCustom}", customRequirementsBuilder.toString());
        return replacer.replace();
    }

    /**
     * Gets a list of item names from the given list of items
     *
     * @param items <p>The items to get the names of</p>
     * @return <p>The names of the given items</p>
     */
    private @NotNull List<String> getItemNames(@NotNull List<?> items) {
        List<String> itemNames = new ArrayList<>();
        for (Object itemStack : items) {
            itemNames.add(QuestsHelper.getUpperCasedItemStackString((ItemStack) itemStack));
        }
        return itemNames;
    }

    /**
     * Gets a list of the quest names for the given quests
     *
     * @param questIds <p>The quests to get names for</p>
     * @return <p>A list of quest names</p>
     */
    private @NotNull List<String> getQuestNames(@NotNull List<String> questIds) {
        List<String> questNames = new ArrayList<>(questIds.size());
        for (String questId : questIds) {
            Quest quest = getQuest(questId);
            if (quest != null) {
                questNames.add(quest.getName());
            }
        }
        return questNames;
    }

    /**
     * Gets the quest with the given id
     *
     * @param questId <p>The id of the quest to get</p>
     * @return <p>The quest, or null if not found</p>
     */
    private @Nullable Quest getQuest(@NotNull String questId) {
        for (Quest quest : questsAPI.getLoadedQuests()) {
            if (quest.getId().equals(questId)) {
                return quest;
            }
        }
        return null;
    }

    /**
     * Gets a string for the given list of requirements
     *
     * @param itemList          <p>The items to display in the list of requirements</p>
     * @param formatMessage     <p>The translatable message describing the list format</p>
     * @param formatPlaceholder <p>The placeholder to replace with the list items</p>
     * @param itemMessage       <p>The translatable message describing each item's format</p>
     * @param itemPlaceholder   <p>The placeholder to replace with each item in the list</p>
     * @return <p>The string corresponding to the given requirement list</p>
     */
    private @NotNull String getRequirementList(@NotNull List<String> itemList, @NotNull TranslatableMessage formatMessage,
                                               @NotNull String formatPlaceholder, @NotNull TranslatableMessage itemMessage,
                                               @NotNull String itemPlaceholder) {
        StringBuilder blockedBuilder = new StringBuilder();
        for (Object requirements : itemList) {
            blockedBuilder.append(formatter.replacePlaceholder(itemMessage, itemPlaceholder,
                    String.valueOf(requirements)));
        }
        return formatter.replacePlaceholder(formatMessage, formatPlaceholder, blockedBuilder.toString());
    }

}
