package net.knarcraft.dynmapcitizens.handler.trait.quests;

import me.pikamug.quests.quests.Quest;
import me.pikamug.quests.quests.components.Stage;
import net.citizensnpcs.api.CitizensAPI;
import net.citizensnpcs.api.npc.NPC;
import net.citizensnpcs.api.npc.NPCRegistry;
import net.knarcraft.dynmapcitizens.util.QuestsHelper;
import org.bukkit.DyeColor;
import org.bukkit.Location;
import org.bukkit.entity.EntityType;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.UUID;

/**
 * A class to generate a string containing all information about a quest's stages
 */
public class QuestStagesInfoGenerator {

    private final Quest quest;

    /**
     * Instantiates a new quest stages info generator
     *
     * @param quest <p>The quest to generate information about</p>
     */
    public QuestStagesInfoGenerator(@NotNull Quest quest) {
        this.quest = quest;
    }

    /**
     * Gets information about a quest's stages
     *
     * @return <p>A string with information about the quest's stages</p>
     */
    public @NotNull String getQuestStagesInfo() {
        StringBuilder questInfo = new StringBuilder();
        NPCRegistry registry = CitizensAPI.getNPCRegistry();
        int stageCounter = 1;

        if (quest.getStages().size() == 1) {
            questInfo.append("<b>Tasks</b><ul>");
            questInfo.append(getStageInfo(quest.getStage(0), registry));
        } else {
            questInfo.append("<b>Stages:</b><ul>");
            for (Stage stage : quest.getStages()) {
                questInfo.append("<li><b>Stage ").append(stageCounter).append(" tasks:</b><ul>");

                questInfo.append(getStageInfo(stage, registry));

                questInfo.append("</ul></li>");
                stageCounter++;
            }
        }
        questInfo.append("</ul>");

        return questInfo.toString();
    }

    /**
     * Gets information about the given stage
     *
     * @param stage    <p>The stage to get information about</p>
     * @param registry <p>The registry to get NPC info from</p>
     * @return <p>Information about the stage's tasks</p>
     */
    private @NotNull String getStageInfo(@NotNull Stage stage, @NotNull NPCRegistry registry) {
        StringBuilder questInfo = new StringBuilder();
        int mobTypes = stage.getMobsToKill().size();
        for (int i = 0; i < mobTypes; i++) {
            questInfo.append("<li>Kill ").append(QuestsHelper.normalizeName(((EntityType) stage.getMobsToKill().get(i)).name()));
            questInfo.append(" x ").append(stage.getMobNumToKill().get(i));
            if (stage.getKillNames().size() > i) {
                questInfo.append(" at ").append(stage.getKillNames().get(i));
            }
            questInfo.append("</li>");
        }
        int reachLocations = stage.getLocationsToReach().size();
        for (int i = 0; i < reachLocations; i++) {
            questInfo.append("<li>Reach ").append(stage.getLocationNames().get(i)).append(", within ");
            questInfo.append(stage.getRadiiToReachWithin().get(i)).append(" blocks of ");
            questInfo.append(getLocationString((Location) stage.getLocationsToReach().get(i))).append("</li>");
        }

        int deliveries = stage.getItemDeliveryTargets().size();
        for (int i = 0; i < deliveries; i++) {
            NPC npc = registry.getByUniqueId(stage.getItemDeliveryTargets().get(i));
            questInfo.append("<li>Deliver ").append(QuestsHelper.getItemStackString((ItemStack) stage.getItemsToDeliver().get(i))).append(
                    " to ").append(npc.getName()).append("</li>");
        }
        if (stage.getFishToCatch() != null) {
            questInfo.append("<li>Catch ").append(stage.getFishToCatch()).append(" fish").append("</li>");
        }
        for (UUID npcId : stage.getNpcsToKill()) {
            questInfo.append("<li>Kill NPC ").append(registry.getByUniqueId(npcId).getName()).append("</li>");
        }
        for (UUID npcId : stage.getNpcsToInteract()) {
            questInfo.append("<li>Talk to ").append(registry.getByUniqueId(npcId).getName()).append("</li>");
        }

        questInfo.append(getQuestItemsTaskString(stage.getBlocksToBreak(), "<li>Break ")).append("</li>");
        questInfo.append(getQuestItemsTaskString(stage.getBlocksToCut(), "<li>Cut ")).append("</li>");
        questInfo.append(getQuestItemsTaskString(stage.getBlocksToDamage(), "<li>Damage ")).append("</li>");
        questInfo.append(getQuestItemsTaskString(stage.getBlocksToUse(), "<li>Use ")).append("</li>");
        questInfo.append(getQuestItemsTaskString(stage.getBlocksToPlace(), "<li>Place ")).append("</li>");

        questInfo.append(getQuestItemsTaskString(stage.getItemsToBrew(), "<li>Brew ")).append("</li>");
        questInfo.append(getQuestItemsTaskString(stage.getItemsToConsume(), "<li>Consume ")).append("</li>");
        questInfo.append(getQuestItemsTaskString(stage.getItemsToCraft(), "<li>Craft ")).append("</li>");
        questInfo.append(getQuestItemsTaskString(stage.getItemsToEnchant(), "<li>Enchant ")).append("</li>");
        questInfo.append(getQuestItemsTaskString(stage.getItemsToSmelt(), "<li>Smelt ")).append("</li>");

        int sheepTypes = stage.getSheepToShear().size();
        for (int i = 0; i < sheepTypes; i++) {
            questInfo.append("<li>Shear ").append(stage.getSheepNumToShear().get(i)).append(" ").append(
                    QuestsHelper.normalizeName(((DyeColor) stage.getSheepToShear().get(i)).name())).append(" sheep").append("</li>");
        }
        if (stage.getCowsToMilk() != null) {
            questInfo.append("<li>Milk ").append(stage.getCowsToMilk()).append(" cows").append("</li>");
        }

        int mobTamingEntries = stage.getMobsToTame().size();
        for (int i = 0; i < mobTamingEntries; i++) {
            questInfo.append("<li>Tame ").append(stage.getMobNumToTame().get(i)).append(" ").append(
                    QuestsHelper.normalizeName(((EntityType) stage.getMobsToTame().get(i)).name())).append("</li>");
        }

        return questInfo.toString();
    }

    /**
     * Gets a user-friendly string-representation of the given location
     *
     * @param location <p>The location to show</p>
     * @return <p>A human-friendly location string</p>
     */
    private @NotNull String getLocationString(@NotNull Location location) {
        String locationString = location.getX() + "," + location.getY() + "," + location.getZ();
        if (location.getWorld() != null) {
            locationString += " in world \"" + location.getWorld().getName() + "\"";
        }
        return locationString;
    }

    /**
     * Gets a string to display a quest task involving some action on an item
     *
     * @param items       <p>The items that are part of the task</p>
     * @param explanation <p>The explanation of what the player needs to do with the items</p>
     * @return <p>A string describing the necessary tasks</p>
     */
    private @NotNull String getQuestItemsTaskString(@NotNull List<?> items, @NotNull String explanation) {
        StringBuilder questInfo = new StringBuilder();
        for (Object itemStack : items) {
            questInfo.append(explanation).append(QuestsHelper.getItemStackString((ItemStack) itemStack));
        }
        return questInfo.toString();
    }

}
