package net.knarcraft.dynmapcitizens.handler.trait.quests;

import me.pikamug.quests.Quests;
import me.pikamug.quests.quests.Quest;
import me.pikamug.quests.quests.components.Stage;
import net.citizensnpcs.api.CitizensAPI;
import net.citizensnpcs.api.npc.NPCRegistry;
import net.knarcraft.dynmapcitizens.DynmapCitizens;
import net.knarcraft.dynmapcitizens.handler.trait.AbstractTraitHandler;
import net.knarcraft.dynmapcitizens.property.Icon;
import net.knarcraft.dynmapcitizens.settings.QuestsSettings;
import net.knarcraft.dynmapcitizens.settings.TraitSettings;
import net.knarcraft.dynmapcitizens.util.QuestsHelper;
import org.bukkit.Bukkit;
import org.dynmap.DynmapAPI;
import org.dynmap.markers.GenericMarker;
import org.dynmap.markers.Marker;
import org.dynmap.markers.MarkerIcon;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * A handler class for the quests trait
 */
public class QuestsHandler extends AbstractTraitHandler {

    private Quests questsAPI;
    private Map<Icon, MarkerIcon> markerIcons;
    private final Map<UUID, NPCQuestInfo> questGiverInfo = new HashMap<>();
    private final QuestsSettings settings = new QuestsSettings();
    private final List<Quest> unavailableQuests = new ArrayList<>();
    private QuestAreaHandler questAreaHandler;

    @Override
    public void initialize() {
        questsAPI = (Quests) Bukkit.getServer().getPluginManager().getPlugin("Quests");
        DynmapAPI dynmapAPI = DynmapCitizens.getInstance().getDynmapAPI();
        markerIcons = DynmapCitizens.getInstance().getGlobalSettings().getMarkerIcons();
        if (questsAPI != null) {
            super.initializeMarkerSet();
            super.markerSet.getMarkers().forEach(GenericMarker::deleteMarker);

            questAreaHandler = new QuestAreaHandler(questsAPI, dynmapAPI, settings, unavailableQuests);
        } else {
            isEnabled = false;
        }
    }

    @Override
    public @NotNull TraitSettings getSettings() {
        return this.settings;
    }

    @Override
    public void updateMarkers() {
        if (questsAPI.isLoading()) {
            return;
        }

        //Generate information about all NPCs involved in quests
        generateQuestNPCInfo();

        //Update all area markers for quests
        questAreaHandler.updateQuestAreas();

        //Remove any markers for deleted quests
        for (Marker marker : super.markerSet.getMarkers()) {
            if (!questGiverInfo.containsKey(UUID.fromString(marker.getMarkerID()))) {
                marker.deleteMarker();
            }
        }

        //Generate markers based on the generated info
        generateAllMarkers();
    }

    /**
     * Generates information about all NPCs involved in quests
     */
    private void generateQuestNPCInfo() {
        //Clear any previous information
        questGiverInfo.clear();
        unavailableQuests.clear();
        //Generation information about NPC's parts in each quest
        for (Quest quest : questsAPI.getLoadedQuests()) {
            if (QuestsHelper.isQuestUnavailable(quest)) {
                unavailableQuests.add(quest);
                //If unavailable quests aren't displayed, there is no point in continuing
                if (settings.hideUnavailableQuests()) {
                    continue;
                }
            }
            if (quest.getNpcStart() != null) {
                getInfo(quest.getNpcStart()).addQuestStart(quest);
            }
            for (Stage stage : quest.getStages()) {
                for (UUID npcId : stage.getNpcsToKill()) {
                    getInfo(npcId).addQuestKill(quest);
                }
                for (UUID npcId : stage.getItemDeliveryTargets()) {
                    getInfo(npcId).addQuestDeliver(quest);
                }
                for (UUID npcId : stage.getNpcsToInteract()) {
                    getInfo(npcId).addQuestInteract(quest);
                }
            }
        }
    }

    /**
     * Generates all quest markers based on the previously generated quest NPC info
     */
    private void generateAllMarkers() {
        NPCRegistry registry = CitizensAPI.getNPCRegistry();
        //Add markers for each NPC detected as part of a quest
        for (UUID npcId : questGiverInfo.keySet()) {
            NPCQuestInfo info = questGiverInfo.get(npcId);
            MarkerIcon icon = markerIcons.get(info.getNPCIcon());
            List<Quest> offeredQuests = info.getQuestStarts();
            StringBuilder markerDescription = new StringBuilder();

            markerDescription.append("<h2>").append(registry.getByUniqueId(npcId).getName()).append("</h2>");

            //Print info about any offered quests
            if (!offeredQuests.isEmpty()) {
                appendOfferedQuestsInfo(markerDescription, offeredQuests);
            }

            markerDescription.append(getInvolvedInQuestsString(info));

            Marker existingMarker = super.markerSet.findMarker(npcId.toString());
            String newDescription = markerDescription.toString();
            if (existingMarker != null) {
                if (!existingMarker.getDescription().equals(newDescription)) {
                    existingMarker.setDescription(newDescription);
                }
            } else {
                addNPCMarker(npcId, info.getQuestNPCType().getMarkerTitle(), newDescription, icon, super.markerSet);
            }
        }
    }

    /**
     * Appends information about an NPC's offered quests to the given string builder
     *
     * @param stringBuilder <p>The string builder to append to</p>
     * @param offeredQuests <p>The list of quests the NPC offers</p>
     */
    private void appendOfferedQuestsInfo(@NotNull StringBuilder stringBuilder, @NotNull List<Quest> offeredQuests) {
        stringBuilder.append("<h3>Quests offered:</h3><ul>");
        for (Quest quest : offeredQuests) {
            stringBuilder.append("<li><h4><b>").append(quest.getName()).append("</b></h4><h5><b>- ");
            stringBuilder.append(quest.getDescription()).append("</b></h5>");
            if (settings.displayStageInfo()) {
                stringBuilder.append(new QuestStagesInfoGenerator(quest).getQuestStagesInfo());
            }
            if (settings.displayRewardInfo()) {
                stringBuilder.append(new QuestRewardsInfoGenerator(quest).getQuestRewardsInfo());
            }
            if (settings.displayRequirementInfo()) {
                stringBuilder.append(new QuestRequirementsInfoGenerator(questsAPI, quest).getQuestRequirementsInfo());
            }
            if (settings.displayPlannerInfo()) {
                stringBuilder.append(new QuestPlannerInfoGenerator(quest).getQuestPlannerInfo());
            }
            stringBuilder.append("</li>");
        }
        stringBuilder.append("</ul>");
    }

    /**
     * Gets information about the quests an NPC is involved in
     *
     * @param info <p>The NPC info to look through</p>
     * @return <p>Information about an NPC's involvement in different quests</p>
     */
    private String getInvolvedInQuestsString(@NotNull NPCQuestInfo info) {
        List<Quest> questKills = info.getQuestKills();
        List<Quest> questInteractions = info.getQuestInteractions();
        List<Quest> questDeliveries = info.getQuestDeliveries();

        if (questKills.isEmpty() && questInteractions.isEmpty() && questDeliveries.isEmpty()) {
            return "";
        }
        StringBuilder markerDescription = new StringBuilder();
        markerDescription.append("<h3>Involved in quests:</h3><ul>");
        addInvolvedInString("Killed in", questKills, markerDescription);
        addInvolvedInString("Delivery target in", questDeliveries, markerDescription);
        addInvolvedInString("Interacted with in quest", questInteractions, markerDescription);
        markerDescription.append("</ul>");
        return markerDescription.toString();
    }

    /**
     * Adds string explaining how an NPC is involved with the given quests
     *
     * @param prefix  <p>The string explaining how the NPC is involved</p>
     * @param quests  <p>The quests the NPC is involved in</p>
     * @param builder <p>The string builder to append to</p>
     */
    private void addInvolvedInString(@NotNull String prefix, @NotNull List<Quest> quests,
                                     @NotNull StringBuilder builder) {
        for (Quest quest : new HashSet<>(quests)) {
            builder.append("<li>").append(prefix).append(": ").append(quest.getName()).append("</li>");
        }
    }


    /**
     * Gets the info object for the given NPC
     *
     * @param npcId <p>The id of the NPC to get information about</p>
     * @return <p>The NPC's info object</p>
     */
    private @NotNull NPCQuestInfo getInfo(@NotNull UUID npcId) {
        if (questGiverInfo.get(npcId) == null) {
            questGiverInfo.put(npcId, new NPCQuestInfo());
        }
        return questGiverInfo.get(npcId);
    }

}
