package net.knarcraft.dynmapcitizens.util;

import net.knarcraft.dynmapcitizens.DynmapCitizens;
import net.knarcraft.dynmapcitizens.settings.AreaMarkerSettings;
import org.bukkit.Location;
import org.bukkit.World;
import org.dynmap.DynmapAPI;
import org.dynmap.markers.CircleMarker;
import org.dynmap.markers.MarkerSet;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.logging.Level;

public class DynmapHelper {

    /**
     * Gets the given marker set (and creates it if necessary)
     *
     * @param dynmapAPI <p>A reference to dynmap's API</p>
     * @param setId     <p>The id of the marker set to get</p>
     * @param label     <p>The label of the marker set if creation is necessary</p>
     * @return <p>The marker set, or null if something went wrong</p>
     */
    public static @Nullable MarkerSet getMarkerSet(@NotNull DynmapAPI dynmapAPI, @NotNull String setId,
                                                   @NotNull String label) {
        MarkerSet markerSet = dynmapAPI.getMarkerAPI().getMarkerSet(setId);
        if (markerSet == null) {
            markerSet = dynmapAPI.getMarkerAPI().createMarkerSet(setId, label, null, false);
            if (markerSet == null) {
                DynmapCitizens.getInstance().getLogger().log(Level.SEVERE, "Unable to get or create " + setId +
                        " marker set");
                DynmapCitizens.getInstance().onDisable();
                return null;
            }
        }
        return markerSet;
    }

    /**
     * Initializes a marker set from the given settings
     *
     * @param dynmapAPI      <p>A reference to the Dynmap API</p>
     * @param markerSettings <p>The settings to use for initialization</p>
     * @return <p>The initialized marker</p>
     */
    public static @Nullable MarkerSet initializeMarkerSet(@NotNull DynmapAPI dynmapAPI,
                                                          @NotNull AreaMarkerSettings markerSettings) {
        MarkerSet markerSet = DynmapHelper.getMarkerSet(dynmapAPI, markerSettings.getMarkerSetId(),
                markerSettings.getMarkerSetName());
        if (markerSet != null) {
            markerSet.setHideByDefault(markerSettings.markersHiddenByDefault());
            markerSet.setLayerPriority(markerSettings.getMarkerSetPriority());
        }
        return markerSet;
    }

    /**
     * Marks a location on the dynamic map
     *
     * @param location       <p>The location to mark</p>
     * @param radius         <p>The radius of the circular area to mark</p>
     * @param description    <p>The description to put on the marker</p>
     * @param markerSet      <p>The marker set to use when marking the location</p>
     * @param markerSettings <p>The settings to use for the marker</p>
     */
    public static void markLocation(@NotNull Location location, @NotNull Integer radius,
                                    @NotNull String description, @NotNull MarkerSet markerSet,
                                    @NotNull AreaMarkerSettings markerSettings) {
        //Skip if location is invalid
        World world = location.getWorld();
        if (world == null) {
            return;
        }

        CircleMarker circleMarker = markerSet.createCircleMarker(null, description, true,
                world.getName(), location.getX(), location.getY(), location.getZ(), radius, radius, false);
        if (circleMarker == null) {
            DynmapCitizens.getInstance().getLogger().log(Level.WARNING, "Unable to create circle marker at " +
                    location + " with radius " + radius);
        } else {
            circleMarker.setFillStyle(markerSettings.getFillOpacity(), getColor(markerSettings.getFillColor()));
            circleMarker.setLineStyle(markerSettings.getLineThickness(), markerSettings.getLineOpacity(),
                    getColor(markerSettings.getLineColor()));
        }
    }

    /**
     * Gets an integer representation from a hexadecimal color code
     *
     * @param color <p>A hexadecimal color</p>
     * @return <p>An integer representation of the color</p>
     */
    private static int getColor(@NotNull String color) {
        try {
            return Integer.parseInt(color, 16);
        } catch (NumberFormatException exception) {
            return 0;
        }
    }

}
